# Architecture


Blueapi performs a number of tasks:

* Managing the Bluesky [RunEngine](https://nsls-ii.github.io/bluesky/run_engine_api.html), giving it instructions and handling its errors. Traditionally this job has been done by a human with an [IPython](https://ipython.org/) terminal, so it requires automating.
* Maintaining a registry of plans and devices. In the aforementioned IPython_ case, these would have just been global variables.
* Communicating with the outside world, accepting instructions to run plans, providing updates on plan progress etc.

These responsibilities are kept separate in the codebase to ensure a clean, maintainable architecture.

## Key Components


![blueapi architecture main components](../images/blueapi-architecture.png)


### The `BlueskyContext` Object

Holds registries of plans and devices as well as a number of helper methods for 
registering en-masse from a normal Python module. 

### The Worker Object

Wraps the Bluesky `RunEngine` and accepts requests to run plans. The requests include the name 
of the plan and a dictionary of parameters to pass. The worker validates the parameters against
the known expectations of the plan, passes it to the `RunEngine` and handles any errors.


### The Service Object

Handles communications and the API layer. This object holds a reference to the worker 
can interrogate it/give it instructions in response to messages it receives from the message
bus. It can also forward the various events generated by the worker to topics on the bus.
